{% extends "dynamic_preferences/base.html" %}
{% load i18n %}
{% block content %}

    {% include "dynamic_preferences/sections.html" with sections=registry.sections %}

    <form action="" enctype="multipart/form-data" method="post">
        {% csrf_token %}
        {{ form.as_p }}
        <input type="submit" value="{% trans 'Submit' %}" />
    </form>
{% endblock %}